MAX_TGW_CNT_PER_PBU = 10
XTP_VERSION_LEN = 16
XTP_TRADING_DAY_LEN = 9
XTP_TICKER_LEN = 16
XTP_TICKER_NAME_LEN = 64
XTP_LOCAL_ORDER_LEN = 11
XTP_ORDER_EXCH_LEN = 17
XTP_EXEC_ID_LEN = 18
XTP_BRANCH_PBU_LEN = 7
XTP_ACCOUNT_NAME_LEN = 16
XTP_CREDIT_DEBT_ID_LEN = 33
XTP_INET_ADDRESS_STR_LEN = 64
XTP_MAC_ADDRESS_LEN = 16
XTP_HARDDISK_SN_LEN = 24
XTP_MACOS_SNO_LEN = 21
XTP_STRATEGE_LEG_NUM = 4
XTP_STRATEGY_ID_LEN = 10
XTP_STRATEGY_NAME_LEN = 32
XTP_SECONDARY_ORDER_ID_LEN = 18
XTP_CNTRT_COMB_STRA_LIST_LEN = 2048
XTP_COMBINED_EXECUTION_LEG_NUM = 2
XTP_LOG_LEVEL_FATAL = 1
XTP_LOG_LEVEL_ERROR = 2
XTP_LOG_LEVEL_WARNING = 3
XTP_LOG_LEVEL_INFO = 4
XTP_LOG_LEVEL_DEBUG = 5
XTP_LOG_LEVEL_TRACE = 6
XTP_PROTOCOL_TCP = 1
XTP_PROTOCOL_UDP = 2
XTP_EXCHANGE_SH = 1
XTP_EXCHANGE_SZ = 2
XTP_EXCHANGE_UNKNOWN = 3
XTP_MKT_INIT = 0
XTP_MKT_SZ_A = 1
XTP_MKT_SH_A = 2
XTP_MKT_UNKNOWN = 3
XTP_PRICE_LIMIT = 1
XTP_PRICE_BEST_OR_CANCEL = 2
XTP_PRICE_BEST5_OR_LIMIT = 3
XTP_PRICE_BEST5_OR_CANCEL = 4
XTP_PRICE_ALL_OR_CANCEL = 5
XTP_PRICE_FORWARD_BEST = 6
XTP_PRICE_REVERSE_BEST_LIMIT = 7
XTP_PRICE_LIMIT_OR_CANCEL = 8
XTP_PRICE_TYPE_UNKNOWN = 9
XTP_SIDE_BUY = 1
XTP_SIDE_SELL = 2
XTP_SIDE_PURCHASE = 7
XTP_SIDE_REDEMPTION = 8
XTP_SIDE_SPLIT = 9
XTP_SIDE_MERGE = 10
XTP_SIDE_COVER = 11
XTP_SIDE_FREEZE = 12
XTP_SIDE_MARGIN_TRADE = 21
XTP_SIDE_SHORT_SELL = 22
XTP_SIDE_REPAY_MARGIN = 23
XTP_SIDE_REPAY_STOCK = 24
XTP_SIDE_STOCK_REPAY_STOCK = 26
XTP_SIDE_SURSTK_TRANS = 27
XTP_SIDE_GRTSTK_TRANSIN = 28
XTP_SIDE_GRTSTK_TRANSOUT = 29
XTP_SIDE_OPT_COMBINE = 31
XTP_SIDE_OPT_SPLIT = 32
XTP_SIDE_OPT_SPLIT_FORCE = 33
XTP_SIDE_OPT_SPLIT_FORCE_EXCH = 34
XTP_SIDE_UNKNOWN = 50
XTP_POSITION_EFFECT_INIT = 0
XTP_POSITION_EFFECT_OPEN = 1
XTP_POSITION_EFFECT_CLOSE = 2
XTP_POSITION_EFFECT_FORCECLOSE = 3
XTP_POSITION_EFFECT_CLOSETODAY = 4
XTP_POSITION_EFFECT_CLOSEYESTERDAY = 5
XTP_POSITION_EFFECT_FORCEOFF = 6
XTP_POSITION_EFFECT_LOCALFORCECLOSE = 7
XTP_POSITION_EFFECT_CREDIT_FORCE_COVER = 8
XTP_POSITION_EFFECT_CREDIT_FORCE_CLEAR = 9
XTP_POSITION_EFFECT_CREDIT_FORCE_DEBT = 10
XTP_POSITION_EFFECT_CREDIT_FORCE_UNCOND = 11
XTP_POSITION_EFFECT_UNKNOWN = 12
XTP_ORDER_ACTION_STATUS_SUBMITTED = 1
XTP_ORDER_ACTION_STATUS_ACCEPTED = 2
XTP_ORDER_ACTION_STATUS_REJECTED = 3
XTP_ORDER_STATUS_INIT = 0
XTP_ORDER_STATUS_ALLTRADED = 1
XTP_ORDER_STATUS_PARTTRADEDQUEUEING = 2
XTP_ORDER_STATUS_PARTTRADEDNOTQUEUEING = 3
XTP_ORDER_STATUS_NOTRADEQUEUEING = 4
XTP_ORDER_STATUS_CANCELED = 5
XTP_ORDER_STATUS_REJECTED = 6
XTP_ORDER_STATUS_UNKNOWN = 7
XTP_ORDER_SUBMIT_STATUS_INSERT_SUBMITTED = 1
XTP_ORDER_SUBMIT_STATUS_INSERT_ACCEPTED = 2
XTP_ORDER_SUBMIT_STATUS_INSERT_REJECTED = 3
XTP_ORDER_SUBMIT_STATUS_CANCEL_SUBMITTED = 4
XTP_ORDER_SUBMIT_STATUS_CANCEL_REJECTED = 5
XTP_ORDER_SUBMIT_STATUS_CANCEL_ACCEPTED = 6
XTP_TERT_RESTART = 0
XTP_TERT_RESUME = 1
XTP_TERT_QUICK = 2
XTP_TICKER_TYPE_STOCK = 0
XTP_TICKER_TYPE_INDEX = 1
XTP_TICKER_TYPE_FUND = 2
XTP_TICKER_TYPE_BOND = 3
XTP_TICKER_TYPE_OPTION = 4
XTP_TICKER_TYPE_TECH_STOCK = 5
XTP_TICKER_TYPE_UNKNOWN = 6
XTP_BUSINESS_TYPE_CASH = 0
XTP_BUSINESS_TYPE_IPOS = 1
XTP_BUSINESS_TYPE_REPO = 2
XTP_BUSINESS_TYPE_ETF = 3
XTP_BUSINESS_TYPE_MARGIN = 4
XTP_BUSINESS_TYPE_DESIGNATION = 5
XTP_BUSINESS_TYPE_ALLOTMENT = 6
XTP_BUSINESS_TYPE_STRUCTURED_FUND_PURCHASE_REDEMPTION = 7
XTP_BUSINESS_TYPE_STRUCTURED_FUND_SPLIT_MERGE = 8
XTP_BUSINESS_TYPE_MONEY_FUND = 9
XTP_BUSINESS_TYPE_OPTION = 10
XTP_BUSINESS_TYPE_EXECUTE = 11
XTP_BUSINESS_TYPE_FREEZE = 12
XTP_BUSINESS_TYPE_OPTION_COMBINE = 13
XTP_BUSINESS_TYPE_EXECUTE_COMBINE = 14
XTP_BUSINESS_TYPE_UNKNOWN = 15
XTP_ACCOUNT_NORMAL = 0
XTP_ACCOUNT_CREDIT = 1
XTP_ACCOUNT_DERIVE = 2
XTP_ACCOUNT_UNKNOWN = 3
XTP_FUND_TRANSFER_OUT = 0
XTP_FUND_TRANSFER_IN = 1
XTP_FUND_INTER_TRANSFER_OUT = 2
XTP_FUND_INTER_TRANSFER_IN = 3
XTP_FUND_TRANSFER_UNKNOWN = 4
XTP_FUND_OPER_PROCESSING = 0
XTP_FUND_OPER_SUCCESS = 1
XTP_FUND_OPER_FAILED = 2
XTP_FUND_OPER_SUBMITTED = 3
XTP_FUND_OPER_UNKNOWN = 4
XTP_DEBT_EXTEND_OPER_PROCESSING = 0
XTP_DEBT_EXTEND_OPER_SUBMITTED = 1
XTP_DEBT_EXTEND_OPER_SUCCESS = 2
XTP_DEBT_EXTEND_OPER_FAILED = 3
XTP_DEBT_EXTEND_OPER_UNKNOWN = 4
XTP_SPLIT_MERGE_STATUS_ALLOW = 0
XTP_SPLIT_MERGE_STATUS_ONLY_SPLIT = 1
XTP_SPLIT_MERGE_STATUS_ONLY_MERGE = 2
XTP_SPLIT_MERGE_STATUS_FORBIDDEN = 3
XTP_TBT_ENTRUST = 1
XTP_TBT_TRADE = 2
XTP_OPT_CALL = 1
XTP_OPT_PUT = 2
XTP_OPT_EXERCISE_TYPE_EUR = 1
XTP_OPT_EXERCISE_TYPE_AME = 2
XTP_POSITION_DIRECTION_NET = 0
XTP_POSITION_DIRECTION_LONG = 1
XTP_POSITION_DIRECTION_SHORT = 2
XTP_POSITION_DIRECTION_COVERED = 3
XTP_POSITION_UNCOVERED = 0
XTP_POSITION_COVERED = 1
XTP_CRD_CR_INIT = 0
XTP_CRD_CR_SUCCESS = 1
XTP_CRD_CR_FAILED = 2
XTP_OPT_POSITION_TYPE_CONTRACT = 0
XTP_OPT_POSITION_TYPE_COMBINED = 1
XTP_ORDER_DETAIL_TYPE_NEW_ORDER = 0
XTP_ORDER_DETAIL_TYPE_CANCEL_ORDER = 1
XTP_ORDER_DETAIL_TYPE_OPT_COMB_NEW_ORDER = 2
XTP_ORDER_DETAIL_TYPE_OPT_COMB_CANCEL_ORDER = 3
XTP_TRDT_COMMON = '0'
XTP_TRDT_CASH = '1'
XTP_TRDT_PRIMARY = '2'
XTP_TRDT_CROSS_MKT_CASH = '3'
XTP_ORDT_Normal = '0'
XTP_ORDT_DeriveFromQuote = '1'
XTP_ORDT_DeriveFromCombination = '2'
XTP_ORDT_Combination = '3'
XTP_ORDT_ConditionalOrder = '4'
XTP_ORDT_Swap = '5'
XTP_TERMINAL_PC = 1
XTP_TERMINAL_ANDROID = 2
XTP_TERMINAL_IOS = 3
XTP_TERMINAL_WP = 4
XTP_TERMINAL_STATION = 5
XTP_TERMINAL_TEL = 6
XTP_TERMINAL_PC_LINUX = 7
XTP_EXP_DATE_SAME = 0
XTP_EXP_DATE_DIFF = 1
XTP_EXP_DATE_NON = 2
XTP_UNDERLYING_SAME = 0
XTP_UNDERLYING_DIFF = 1
XTP_UNDERLYING_NON = 2
XTP_AUTO_SPLIT_EXPDAY = 0
XTP_AUTO_SPLIT_PREDAY = 1
XTP_AUTO_SPLIT_PRE2DAY = 2
XTP_AUTO_SPLIT_NON = 3
XTP_QUALIFICATION_PUBLIC = 0
XTP_QUALIFICATION_COMMON = 1
XTP_QUALIFICATION_ORGANIZATION = 2
XTP_QUALIFICATION_UNKNOWN = 3
XTP_SECURITY_MAIN_BOARD = 0
XTP_SECURITY_SECOND_BOARD = 1
XTP_SECURITY_STARTUP_BOARD = 2
XTP_SECURITY_INDEX = 3
XTP_SECURITY_TECH_BOARD = 4
XTP_SECURITY_STATE_BOND = 5
XTP_SECURITY_ENTERPRICE_BOND = 6
XTP_SECURITY_COMPANEY_BOND = 7
XTP_SECURITY_CONVERTABLE_BOND = 8
XTP_SECURITY_NATIONAL_BOND_REVERSE_REPO = 12
XTP_SECURITY_ETF_SINGLE_MARKET_STOCK = 14
XTP_SECURITY_ETF_INTER_MARKET_STOCK = 15
XTP_SECURITY_ETF_SINGLE_MARKET_BOND = 17
XTP_SECURITY_ETF_GOLD = 19
XTP_SECURITY_STRUCTURED_FUND_CHILD = 24
XTP_SECURITY_SZSE_RECREATION_FUND = 26
XTP_SECURITY_STOCK_OPTION = 29
XTP_SECURITY_ETF_OPTION = 30
XTP_SECURITY_ALLOTMENT = 100
XTP_SECURITY_MONETARY_FUND_SHCR = 110
XTP_SECURITY_MONETARY_FUND_SHTR = 111
XTP_SECURITY_MONETARY_FUND_SZ = 112
XTP_SECURITY_OTHERS = 255
