.def("queryAllTickers", &MdApi::queryAllTickers)
.def("queryTickersPriceInfo", &MdApi::queryTickersPriceInfo)
.def("queryAllTickersPriceInfo", &MdApi::queryAllTickersPriceInfo)
.def("queryAllTickersFullInfo", &MdApi::queryAllTickersFullInfo)

.def("onDisconnected", &MdApi::onDisconnected)
.def("onError", &MdApi::onError)
.def("onSubMarketData", &MdApi::onSubMarketData)
.def("onUnSubMarketData", &MdApi::onUnSubMarketData)
.def("onDepthMarketData", &MdApi::onDepthMarketData)
.def("onSubOrderBook", &MdApi::onSubOrderBook)
.def("onUnSubOrderBook", &MdApi::onUnSubOrderBook)
.def("onOrderBook", &MdApi::onOrderBook)
.def("onSubTickByTick", &MdApi::onSubTickByTick)
.def("onUnSubTickByTick", &MdApi::onUnSubTickByTick)
.def("onTickByTick", &MdApi::onTickByTick)
.def("onSubscribeAllMarketData", &MdApi::onSubscribeAllMarketData)
.def("onUnSubscribeAllMarketData", &MdApi::onUnSubscribeAllMarketData)
.def("onSubscribeAllOrderBook", &MdApi::onSubscribeAllOrderBook)
.def("onUnSubscribeAllOrderBook", &MdApi::onUnSubscribeAllOrderBook)
.def("onSubscribeAllTickByTick", &MdApi::onSubscribeAllTickByTick)
.def("onUnSubscribeAllTickByTick", &MdApi::onUnSubscribeAllTickByTick)
.def("onQueryAllTickers", &MdApi::onQueryAllTickers)
.def("onQueryTickersPriceInfo", &MdApi::onQueryTickersPriceInfo)
.def("onSubscribeAllOptionMarketData", &MdApi::onSubscribeAllOptionMarketData)
.def("onUnSubscribeAllOptionMarketData", &MdApi::onUnSubscribeAllOptionMarketData)
.def("onSubscribeAllOptionOrderBook", &MdApi::onSubscribeAllOptionOrderBook)
.def("onUnSubscribeAllOptionOrderBook", &MdApi::onUnSubscribeAllOptionOrderBook)
.def("onSubscribeAllOptionTickByTick", &MdApi::onSubscribeAllOptionTickByTick)
.def("onUnSubscribeAllOptionTickByTick", &MdApi::onUnSubscribeAllOptionTickByTick)
.def("onQueryAllTickersFullInfo", &MdApi::onQueryAllTickersFullInfo)
;
