.def("queryOrderByXTPID", &TdApi::queryOrderByXTPID)
.def("queryOrders", &TdApi::queryOrders)
.def("queryUnfinishedOrders", &TdApi::queryUnfinishedOrders)
.def("queryOrdersByPage", &TdApi::queryOrdersByPage)
.def("queryTradesByXTPID", &TdApi::queryTradesByXTPID)
.def("queryTrades", &TdApi::queryTrades)
.def("queryTradesByPage", &TdApi::queryTradesByPage)
.def("queryPosition", &TdApi::queryPosition)
.def("queryAsset", &TdApi::queryAsset)
.def("queryStructuredFund", &TdApi::queryStructuredFund)
.def("queryFundTransfer", &TdApi::queryFundTransfer)
.def("queryETF", &TdApi::queryETF)
.def("queryETFTickerBasket", &TdApi::queryETFTickerBasket)
.def("queryIPOInfoList", &TdApi::queryIPOInfoList)
.def("queryIPOQuotaInfo", &TdApi::queryIPOQuotaInfo)
.def("queryOptionAuctionInfo", &TdApi::queryOptionAuctionInfo)
.def("queryCreditCashRepayInfo", &TdApi::queryCreditCashRepayInfo)
.def("queryCreditFundInfo", &TdApi::queryCreditFundInfo)
.def("queryCreditDebtInfo", &TdApi::queryCreditDebtInfo)
.def("queryCreditTickerDebtInfo", &TdApi::queryCreditTickerDebtInfo)
.def("queryCreditAssetDebtInfo", &TdApi::queryCreditAssetDebtInfo)
.def("queryCreditTickerAssignInfo", &TdApi::queryCreditTickerAssignInfo)
.def("queryCreditExcessStock", &TdApi::queryCreditExcessStock)
.def("queryMulCreditExcessStock", &TdApi::queryMulCreditExcessStock)
.def("queryCreditExtendDebtDateOrders", &TdApi::queryCreditExtendDebtDateOrders)
.def("queryCreditFundExtraInfo", &TdApi::queryCreditFundExtraInfo)
.def("queryCreditPositionExtraInfo", &TdApi::queryCreditPositionExtraInfo)
.def("queryOptionCombinedUnfinishedOrders", &TdApi::queryOptionCombinedUnfinishedOrders)
.def("queryOptionCombinedOrderByXTPID", &TdApi::queryOptionCombinedOrderByXTPID)
.def("queryOptionCombinedOrders", &TdApi::queryOptionCombinedOrders)
.def("queryOptionCombinedOrdersByPage", &TdApi::queryOptionCombinedOrdersByPage)
.def("queryOptionCombinedTradesByXTPID", &TdApi::queryOptionCombinedTradesByXTPID)
.def("queryOptionCombinedTrades", &TdApi::queryOptionCombinedTrades)
.def("queryOptionCombinedTradesByPage", &TdApi::queryOptionCombinedTradesByPage)
.def("queryOptionCombinedPosition", &TdApi::queryOptionCombinedPosition)
.def("queryOptionCombinedStrategyInfo", &TdApi::queryOptionCombinedStrategyInfo)
.def("queryOptionCombinedExecPosition", &TdApi::queryOptionCombinedExecPosition)

.def("onDisconnected", &TdApi::onDisconnected)
.def("onError", &TdApi::onError)
.def("onOrderEvent", &TdApi::onOrderEvent)
.def("onTradeEvent", &TdApi::onTradeEvent)
.def("onCancelOrderError", &TdApi::onCancelOrderError)
.def("onQueryOrder", &TdApi::onQueryOrder)
.def("onQueryOrderByPage", &TdApi::onQueryOrderByPage)
.def("onQueryTrade", &TdApi::onQueryTrade)
.def("onQueryTradeByPage", &TdApi::onQueryTradeByPage)
.def("onQueryPosition", &TdApi::onQueryPosition)
.def("onQueryAsset", &TdApi::onQueryAsset)
.def("onQueryStructuredFund", &TdApi::onQueryStructuredFund)
.def("onQueryFundTransfer", &TdApi::onQueryFundTransfer)
.def("onFundTransfer", &TdApi::onFundTransfer)
.def("onQueryETF", &TdApi::onQueryETF)
.def("onQueryETFBasket", &TdApi::onQueryETFBasket)
.def("onQueryIPOInfoList", &TdApi::onQueryIPOInfoList)
.def("onQueryIPOQuotaInfo", &TdApi::onQueryIPOQuotaInfo)
.def("onQueryOptionAuctionInfo", &TdApi::onQueryOptionAuctionInfo)
.def("onCreditCashRepay", &TdApi::onCreditCashRepay)
.def("onCreditCashRepayDebtInterestFee", &TdApi::onCreditCashRepayDebtInterestFee)
.def("onQueryCreditCashRepayInfo", &TdApi::onQueryCreditCashRepayInfo)
.def("onQueryCreditFundInfo", &TdApi::onQueryCreditFundInfo)
.def("onQueryCreditDebtInfo", &TdApi::onQueryCreditDebtInfo)
.def("onQueryCreditTickerDebtInfo", &TdApi::onQueryCreditTickerDebtInfo)
.def("onQueryCreditAssetDebtInfo", &TdApi::onQueryCreditAssetDebtInfo)
.def("onQueryCreditTickerAssignInfo", &TdApi::onQueryCreditTickerAssignInfo)
.def("onQueryCreditExcessStock", &TdApi::onQueryCreditExcessStock)
.def("onQueryMulCreditExcessStock", &TdApi::onQueryMulCreditExcessStock)
.def("onCreditExtendDebtDate", &TdApi::onCreditExtendDebtDate)
.def("onQueryCreditExtendDebtDateOrders", &TdApi::onQueryCreditExtendDebtDateOrders)
.def("onQueryCreditFundExtraInfo", &TdApi::onQueryCreditFundExtraInfo)
.def("onQueryCreditPositionExtraInfo", &TdApi::onQueryCreditPositionExtraInfo)
.def("onOptionCombinedOrderEvent", &TdApi::onOptionCombinedOrderEvent)
.def("onOptionCombinedTradeEvent", &TdApi::onOptionCombinedTradeEvent)
.def("onCancelOptionCombinedOrderError", &TdApi::onCancelOptionCombinedOrderError)
.def("onQueryOptionCombinedOrders", &TdApi::onQueryOptionCombinedOrders)
.def("onQueryOptionCombinedOrdersByPage", &TdApi::onQueryOptionCombinedOrdersByPage)
.def("onQueryOptionCombinedTrades", &TdApi::onQueryOptionCombinedTrades)
.def("onQueryOptionCombinedTradesByPage", &TdApi::onQueryOptionCombinedTradesByPage)
.def("onQueryOptionCombinedPosition", &TdApi::onQueryOptionCombinedPosition)
.def("onQueryOptionCombinedStrategyInfo", &TdApi::onQueryOptionCombinedStrategyInfo)
.def("onQueryOptionCombinedExecPosition", &TdApi::onQueryOptionCombinedExecPosition)
;
